<?php

namespace Drupal\monarch_entity_yml;

use Drupal\Core\Config\FileStorage;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Defines import actions for the install profile.
 */
class Import {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Error handling mode.
   *
   * @var string
   */
  public $mode = 'throw';

  /**
   * Create an Import helper class.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Import pre-parsed entity data as entities.
   *
   * @param array $data
   *   The pre-parsed entity data.
   * @param array $feedback
   *   Existing array of entity descriptors.
   *
   * @return array
   *   Array of entity descriptors created.
   */
  public function importData(array $data, array &$feedback = []) {
    foreach ($data as $definition) {
      if (!isset($definition['storage']) || !is_string($definition['storage'])) {
        throw new \Exception('Storage not specified for entity entry.');
      }

      if (!isset($definition['values']) || !is_array($definition['values']) || empty($definition['values'])) {
        throw new \Exception('No valid values for entity entry.');
      }

      try {
        $entity_definition = $this->entityTypeManager->getDefinition($definition['storage']);
        $lookup = [];
        $entity = NULL;
        $created = TRUE;

        foreach ($entity_definition->getKeys() as $key_type => $key) {
          if (in_array($key_type, [
            'id',
            'revision',
          ]) && ($definition['values'][$key] ?? NULL)) {
            $lookup[$key] = $definition['values'][$key];

            while (is_array($lookup[$key])) {
              $lookup[$key] = reset($lookup[$key]);
            }
          }
        }

        if (!empty($lookup)) {
          $entities = $this->entityTypeManager->getStorage($definition['storage'])->loadByProperties($lookup);
          if (count($entities) > 1) {
            throw new \Exception("Unable to import because of multiple entities that already exist matching:\n" . print_r($lookup, TRUE));
          }
          $entity = reset($entities);

          foreach ($definition['values'] as $key => $value) {
            $entity->get($key)->setValue($value);
          }
          $created = FALSE;
        }

        $entity = $entity ?: $this->entityTypeManager->getStorage($definition['storage'])->create($definition['values']);
        $entity->save();
        $feedback[] = '[' . ($entity->uuid() ?? 'No UUID') . ', ' . ($created ? 'create' : 'update') . '] ' . $entity->getEntityTypeId();
      }
      catch (\Throwable $e) {
        if ($entity) {
          $msg = 'Error importing ' . $definition['storage'] . ':' . ($definition['values']['uuid'][0]['value'] ?? 'no_UUID') . ' - ' . $e->getMessage();

          if ($this->mode === 'continue') {
            $feedback[] = $msg;
            $feedback[] = 'Continuing despite error...';
          }
          else {
            throw new \Exception($msg);
          }
        }
        else {
          throw $e;
        }
      }
    }

    return $feedback;
  }

  /**
   * Generates baseline content (intended for after config-import).
   *
   * @param string $path
   *   The path to import files from.
   * @param array $feedback
   *   Existing array of entity descriptors.
   *
   * @return array
   *   Array of entity descriptors created.
   */
  public function import(string $path, array &$feedback = []) {
    $storage = new FileStorage(
      $path
    );

    $files = $storage->readMultiple($storage->listAll(''));
    $feedback = [];

    foreach ($files as $filename => $data) {
      if ($filename[0] !== '_') {
        foreach ($data as &$entry) {
          if (is_string($entry)) {
            $entry = $files[$entry] ?? [];
          }
        }

        $this->importData($data, $feedback);
      }
    }

    return $feedback;
  }

}
